<?php

/*
 * Created on September 5th 2006.
 * 
 * This script is under the MIT license:
 * 
 * Copyright (c) 2006 alias47@hushmail.com
 * Copyright (c) 2009 Michael Tyson <mike@tyson.id.au>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of 
 * this software and associated documentation files (the "Software"), to deal in the Software 
 * without restriction, including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons 
 * to whom the Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in all copies 
 * or substantial portions of the Software.
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
 * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

 //this function takes in the HTML output from the cURL response as a string
 function initial_parse($html) 
 {

	// DEBUG: 
	// Only for tresting from local file
	// $html = file_get_contents('test.html'); //returns string
	//

	// the only variables I can see that might be dynamic and that we would have to
	// retrieve upon each refresh of the page.
	// The challenge much more so than the 'noerror' but let's put that in for good measure.
	$challende = '';
	$noerror = '';

	$dom = new DomDocument;
	@ $dom->loadHTML($html);
	$xpath = new domxpath($dom);

	//get the challenge hash key
	$challenge = $xpath->evaluate('string(//input[@type="hidden" and @name="challenge"]/@value)');
	
	//get the noerror variable
	$noerror = $xpath->evaluate('string(//input[@type="hidden" and @name="noerror"]/@value)');
	
	return array (
		"challenge" => $challenge,
		"noerror" => $noerror
	);

 }

 function create_feed($html)
 {
	//We're going to create a little output buffer called $out
	//let's output some initial XML that is needed for the RSS feed
	//I'll also note now that not knowing a lot about RSS, I just chose 2.0 because it seems
	//widely supported and of course larger version numbers are ALWAYS better. Always. ;)
 	$out = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n";
 	$out .= "<?xml-stylesheet type=\"text/css\" href=\"http://static.ak.fbcdn.net/rsrc.php/z1S1L/lpkg/89khwygv/en_US/141/143242/css/1hb05gk86nr44wko.pkg.css\"?>\r\n";
 	$out .= "<rss version=\"2.0\">\r\n";
 	
 	// DEBUG: 
	// Only for tresting from local file
	// $html = file_get_contents('raw.html'); //returns string
	//
	
	//first let's massage the raw data so all images can be visible; we'll do this by
	//creating absolute URLs from relative ones	
	//you may notice that I took a really half-assed approach to these relative->absolute
	//conversion regular expressions. I'm tired...
	$look_fors = array("/img src=\"\/images/", 
	"/href=\"\//", 
	"/href=\"stories/",
	"/href=\"photo/",
	"/href=\"profile/");
	
	$replace_withs = array("img src=\"http://facebook.com/images", 
	"href=\"http://facebook.com/", 
	"href=\"http://facebook.com/stories",
	"href=\"http://facebook.com/photo",
	"href=\"http://facebook.com/profile" );
	
	$html = preg_replace($look_fors, $replace_withs, $html);	
	
	//now put it into a DOM so we can use DOM-XML to transverse it and find/extract what we
	//need to build an RSS feed from it
	$dom = new DomDocument;
	@ $dom->loadHTML($html);
	$xpath = new domxpath($dom);
	
	//get the your first name
	$name = $xpath->evaluate('string(//li[@id="fb_menu_account"]/div/a)');
	
	//channel link: http://uic.facebook.com/home.php
	//I can't think of any other way to get at the page really
	$channel_link = "http://uic.facebook.com/home.php";
	$channel_title = $name."'s Facebook News Feed";
	$channel_desc = $name."'s Facebook News Feed";
	$channel_lang = "en-us";

	//output the channel info that is needed
	$out .=	"<channel>\r\n";
	$out .=	"<title>".$channel_title."</title>
			<link>".$channel_link."</link>
			<description>".$channel_desc."</description>
			<language>".$channel_lang."</language>\r\n";
	
	//Not going to output this field; I know it is useful but it is not required for 
	//RSS 2.0 and I don't have time to implement caching and such here
	//$out .=	"<lastBuildDate>".date(DATE_RFC822)."</lastBuildDate>";
	
	$entries = $xpath->evaluate('//div[contains(@class, "tab_top_stories")]//div[contains(@class,"feed_item")]');
	
	//loop through the entries identified by the div's above and their xpaths
	for($i=0; $i<$entries->length; $i++)
	{	
		$result_node = $entries->item($i);
		
		// Convert the node to text to use as the description
		$new_dom = new DOMDocument;
		$new_dom->appendChild($new_dom->importNode($result_node,1));
		$entry_desc = $new_dom->saveXML();
		
		//take out the XML identifier that is thrown into
		$entry_desc = str_replace('<?xml version="1.0"?>', '', $entry_desc);
        		
		// Try a few paths to get the item title
		$title = 'Update';
		$title_paths = array('.//span[contains(@class, "status_body")]',
		                     './/div[contains(@class, "header_title_wrapper")]');
		foreach ( $title_paths as $path ) {
		    $title_node_list = $xpath->query($path, $result_node);
		    
		    if ( $title_node_list->length > 0 ) {
		        $title = trim($title_node_list->item(0)->textContent);
		        break;
	        }
        }
        
        if ( !$title ) $title = $result_node->textContent;

		
        // Grab the time from an onclick field
        // This isn't the time of the event at all - I don't actually know what the time here is.
        // As I can't find any way to pick up the time for a particular event, this will do
        $time_node = $xpath->evaluate('.//a[contains(@onclick, "ft(")]', $result_node);
		$time = $time_node->item(0)->getAttribute('onclick');
		if ( preg_match('/"([0-9]{10}):/', $time, $matches) ) {
		    $time = $matches[1];
		} else {
		    $time = time();
		}
        $pubdate = date(DATE_RFC822, $time);
		
		$entry_link = $xpath->evaluate('.//div[contains(@class, "feed_story_wrapper")]//a[@onclick]/@href', $result_node)->item(0)->textContent;
		
		$entry_guid = $entry_link;
		    	
		$out .= "<item>
				<title>".$title."</title>
				<link>".$entry_link."</link>
				<guid>".$entry_guid."</guid>
				<pubDate>".$pubdate."</pubDate>
				<description><![CDATA[".str_replace("]]>", "]]&gt;", $entry_desc)."]]></description>
				</item>";

	}
	
	//close the channel and rss nodes created above
	$out .= "</channel>".
			"</rss>";
	
	//tell all who see this that it is an XML page
	header('Content-Type: text/xml');		
	
	//now we output the $out buffer to display the feed
	echo $out;
 } 
 
?>
